#if WD
/* nothing */
#else
/* Ask various attributes of graphics and text windows */

#include "common.h"
#include "datalib.h"
#include "diagram.h"
#include "curve.h"
#include "visual.h"
#include "attrib.h"
#include "large.h"

extern Global(CharPtr PNTR) AttribTxt;	/* defined in text.c */

enum {		/* see setup.c, init[], /attrib */
  GWINATTR,		/*  Attributes */
  TWINATTR,		/*  Attributes */
  CWINATTR,		/*  Attributes */
  GASIZES,		/* 4,11,(6,10) ! NumLen,TextLen,(ListLines,LineLen) */
  TASIZES,		/* 20,(6,10) ! TextLen,(ListLines,LineLen) */
  GAPOINTS_P,
  GATRACES_P,
  GAPATHS_P,
  GASHIFTS_P,
  GAATTR_P,
  GAOPT_P,
  GAMIN_P,
  GAMAX_P,
  GATICK_P,
  GASCALE_P,
  GAFUNC_P,
  GAGRID_P,
  GAAXIS_P,
  GAABS_P,
  GAORD_P,
  GACIRCLE_P,
  GAWINSAMP_P,
  GASEG_P,
  GASPF_P,
  GASPT_P,
  GAFP,			/* The first point */
  GALP,			/* The last point */
  GACURVES_P,
  GAADDFUN_B,
  GADELFUN_B,
  GAAN_P,
  GAAA_B,
  GADA_B,
  GAJOIN_T,
  GAATTROF_P,
  GAWIDTH_P,
  GASTYLE_P,
  GASOLID_P,
  GADOTTED_P,
  GADASHED_P,
  GACOLORAS_P,
  GAOPTIONS_P,
  GACLEAR_P,
  GAJOINSHIFT_P,
  WINTITLE_P,
  PALETTE_P,
  COL_P,
  FG_P,
  BG_P,
  SHOWPAL_P,
  TAHISTL_P,
  TAHISTB_P,
  TAHISTE_P,
  TALIST_P,
  TAFORMAT_P,
  TATITLE_P,
  TAMSG_B,
  COFORM_T,		/* Format */
  COPREC_P,		/* Precision */
  COSIZES,		/* 4 ! NumLen */
  COALIGN_B		/* Align right */
} AttribTxtIndex;

/*---------------------------------------------------------*/
/* Common routines used both by Text and Graphic functions */

/* Spacing and Margins for normal and hidden groups in Options windows */
#define nMX (sysCharWidth2)
#define nMY (sysLineHeight4)
#define nSX (sysCharWidth)
#define nSY (sysLineHeight4)
#define hMX 0
#define hMY 0
#define hSX (sysCharWidth2)
#define hSY (sysLineHeight4)

Local(GrouP) CreateNormalGroup(GrouP parent, Int2 x, Int2 y, Int2 title) {
  GrouP g;
  g=NormalGroup(parent,x,y,AttribTxt[title],SystemFont,NULL);
  SetGroupMargins(g,nMX,nMY);
  SetGroupSpacing(g,nSX,nSY);
  return g;
}

Local(GrouP) CreateHiddenGroup(GrouP parent, Int2 x, Int2 y, Int2 vspace) {
  GrouP g;
  g=HiddenGroup(parent,x,y,NULL);
  SetGroupMargins(g,hMX,hMY);
  SetGroupSpacing(g,hSX,hSY*vspace);
  return g;
}

Local(Boolean) StdPrmt=TRUE;

/* Creates a StaticPrompt */
Local(PrompT) CreatePrompt(GrouP g, Int2 key) {
  return StaticPrompt(g,key>=0 ? AttribTxt[key] : "",0,StdPrmt ? dialogTextHeight : 0,NULL,'l');
}

Local(WindoW) CreateAttrWin(Int2 wi, Int2 datasize) {
  WindoW w;
  CommonDataPtr pa;
  w=FixedWindow(-50,-50,-sysCharWidth,-sysLineHeight,AttribTxt[wi],NULL);
  pa=_MemNew(datasize);
  SetWindowExtra(w,pa,NULL);
  return w;
}

/* NameList functions */

Local(Int2) ElemVar;	/* var index in the class */
Local(Int2) ElemIndex;	/* running index of list item */
Local(Int2) ElemFound;

Local(CharPtr) CreateName(CharPtr name, Int2 dim) {
  CharPtr p;
  if (dim>1) {
    p=_MemNew(StrLen(name)+5+8+1);
/***********
    sprintf(p,"%s[0..%i]",name,(int)dim-1);
**********/
    sprintf(p,"%s[%i]",name,(int)ElemIndex);
  } else p=_StringSave(name);
  return p;
}

Local(void) ProcessVar(LisT l, CharPtr VarName, Int2 Dim) {
  if (ElemFound) return;
  if (ElemIndex>Dim) {ElemVar++; ElemIndex-=Dim; return;}
  ElemIndex--;
  ElemFound=TRUE;
}

Local(void) NamesListProc(LisT l,Int2 var) {
  CharPtr p;
  CharPtr t;
  CharPtr name;
  CommonDataPtr pa;
  Int2 dim,len,b,e;
  pa=GetWindowExtra(ParentWindow(l));
  if (pa->teInsertTo) {
/*********
    GetNameAndDim(ListCurrentClass,var,&name,&dim);
*********/
    /* var is an ordinal (from 0) number of LisT item selected */
    ElemFound=FALSE;
    ElemIndex=var;
    ElemVar=1;
    EnumVariables(l,ListCurrentClass,ProcessVar);
    GetNameAndDim(ListCurrentClass,ElemVar,&name,&dim);
    p=CreateName(name,dim);
    len=(Int2)TextLength(pa->teInsertTo)+(Int2)StrLen(p)+1;
    t=_MemNew(len);
    GetTitle(pa->teInsertTo,t,len);
    TextSelectionRange(pa->teInsertTo,&b,&e);
    switch (pa->WinPtr->Class) {
      case TWIN:
	if (b>0)
	  if (t[b-1]!=',') {t=StrInsert(t,b,", "); b+=2;}
	t=StrInsert(t,b,p);
	break;
      case GWIN:
	StrCpy(t,p);
	b=0;
	break;
    }
    SetTitle(pa->teInsertTo,t);
    len=b+(Int2)StrLen(p);
    SelectText(pa->teInsertTo,len,len);
    _MemFree(t);
    _MemFree(p);
    pa->FuncChange(pa->teInsertTo);	/* inform it text has been changed */
  }
}

/* Just set teInsert for NamesListProc (see above)*/
Local(void) MarkSelection(TexT t) {
  ((CommonDataPtr)GetWindowExtra(ParentWindow(t)))->teInsertTo=t;
}

/***********************/
/* Graphics attributes */
/***********************/

/* Stores data while attribute input/change */
typedef struct {
  CommonData Common;	/* Common data for all classes of windows */
  GrouP grColor;	/* Color Group (for axes only) */
  struct {
    CharPtr PNTR pTrace;	/* to array of ptrs to all V-functions along the axis */
    Boolean Prop[6];		/* must be == to DIM(GraphicWindow.Axis.Prop) */
    TexT teText[4];
#define teMin	teText[0]	/* Min */
#define teMax	teText[1]	/* Max */
#define teFunc	teText[2]	/* V-Func (currently available for editing) */
#define teTick	teText[3]	/* Tick */
    ButtoN cbProp[6];		/* must be == to DIM(GraphicWindow.Axis.Prop) */
  } Axis[2];
  FuncAttrPtr Attr;		/* to array of points,traces,paths atributes */
  Int2 CurPair;			/* current V-function pair */
  Int2 NumPair;			/* number of pairs */
  ButtoN buDelFun;
  BaR scBar;			/* scroll bar for V-func pair selection */
  GrouP grAttributes;		/* 'Attributes of ...' group */
  GrouP grAttrOf;		/* 'Set attributes of' group */
  ButtoN rbAttrOf[4];		/* CheckBoxes 'Attributes of */
  ButtoN cbJoin[3];		/* 'Join along traces/paths/shifts' buttons */
  ButtoN cbClear;		/* 'Clear before each point' button */
  ButtoN cbPalette;		/* 'Show palette in window' */
  ButtoN cbCircle;		/* 'Draw unit circle in window' */
  GrouP grStyle;		/* 'Style' group */
  GrouP grWidth;		/* 'Width' group */
  PaneL paSample;		/* Panel to display a sample of window */
  PaneL paLimits,paPalette,paDraw;	/* and its superimposed parts */
  ButtoN buColorAs[4];		/* 'Set color the same as' buttons */
  /* Fields related to attributes of current curve */
#if MYSWITCH
  Switch swSegment;		/* Current curve's attrs: 'segment numer' SwitcH */
#else
  SwitcH swSegment;		/* Current curve's attrs: 'segment numer' SwitcH */
#endif
  PrompT prFrom;		/* types of endpoints for current segment */
  PrompT prTo;
#if MYSWITCH
  Switch swAttr;		/* current attributes (CurPair) */
#else
  SwitcH swAttr;		/* current attributes (CurPair) */
#endif
  Int2 spnum,spcur;		/* number of special points and current */
  DataLibPosPtr splist;		/* list of DataLib ptrs to special points */
  ByteStorePtr PNTR spbs;	/* ByteStorePtrs for attrs for all curve's segments */
  DataLibPtr dlsave;		/* store current pos in dl until Ok/Cancel */
  /* End of fields related to attributes of current curve */
  Int2  SampleHeight;
  ShowOptions Options;
  Uint1 Kind;			/* 0-current, 1-axes, 2-curve */
  Uint1 Lock;			/* to prevent recursion in SampleProc */
} GrAttrData, PNTR GrAttrDataPtr;

/* A current V-function has been changed */
Local(void) GrFuncChanged(TexT t) {
  WindowDataPtr pwd;
  GrAttrDataPtr pa;
  pa=(GrAttrDataPtr)GetWindowExtra(ParentWindow(t));
  pwd=pa->Common.WinPtr;
  pwd->flags|=WFL_FUN_CHANGED;
  GetTitleAndSave(t,&pa->Axis[t==pa->Axis[0].teFunc ? 0 : 1].pTrace[pa->CurPair]);
}

Local(void) GrSampleProc(PaneL sample);

/* Show current curve's attributes */
Local(void) ShowAttributes(GrAttrDataPtr pa, Boolean redraw) {
#if !_UNIX
  Int2 p;
#endif
  DrawAttr da;
  Int2 i,j,n;
  Boolean b;
  switch (pa->Kind) {
    case 0:		/* current */
    case 2:		/* curve */
      i=GetValue(pa->grAttrOf);
      if (!Visible(pa->rbAttrOf[i-1]))	/* Currently displayed attr are hidden ==> */
	SetValue(pa->grAttrOf,1);	/*   display points attr. */
      if (pa->Kind==0) {		/* current */
	for (i=0; i<DIM(pa->Axis); i++)
	  SetTitle(pa->Axis[i].teFunc,pa->Axis[i].pTrace[pa->CurPair]);
      }
      SetStatus(pa->cbJoin[0],pa->Attr[pa->CurPair].Trace.style>0);
      SetStatus(pa->cbJoin[1],pa->Attr[pa->CurPair].Path.style>0);
      if (pa->Kind==0)			/* current */
	SetStatus(pa->cbJoin[2],pa->Options.Shift.style>0);
      switch (j=GetValue(pa->grAttrOf)) {
	case 1:		/* Points */
#if !_UNIX
	  p=GAPOINTS_P;
#endif
	  da=pa->Attr[pa->CurPair].Point;
	 l:
	  (j>1 && da.style>0 ? Show : Hide)(pa->grStyle);
	  SetValue(pa->grStyle,MAX(da.style,0));
	  SetValue(pa->grWidth,da.width+1);
	  break;
	case 2:		/* Traces */
#if !_UNIX
	  p=GATRACES_P;
#endif
	  da=pa->Attr[pa->CurPair].Trace;
	  goto l;
	case 3:		/* Paths */
#if !_UNIX
	  p=GAPATHS_P;
#endif
	  da=pa->Attr[pa->CurPair].Path;
	  goto l;
	case 4:		/* Shifts */
#if !_UNIX
	  p=GASHIFTS_P;
#endif
	  da=pa->Options.Shift;
	  goto l;
      }
#if _UNIX
      /* Vibrant under Motif does not support SetTitle for groups */
#else
      StrCpy(ParBuf,AttribTxt[GAATTR_P]);
      StrCat(ParBuf,AttribTxt[p]);
      SetTitle(pa->grAttributes,ParBuf);
#endif
      n=DIM(pa->cbJoin);
      if (pa->Kind==0) {		/* current */
	CorrectBarValue(pa->scBar,pa->CurPair);
	SelectText(pa->Common.teInsertTo=pa->Axis[0].teFunc,0,0);
	(pa->NumPair>1 ? Show : Hide)(pa->cbJoin[2]);	/* Join along shift */
	(pa->NumPair>1 ? Show : Hide)(pa->rbAttrOf[3]);	/* shifts */
      } else {
#if MYSWITCH
	SwitchSetParams(pa->swAttr,pa->CurPair+1,pa->NumPair);  
#else
	SetSwitchParams(pa->swAttr,pa->CurPair+1,pa->NumPair);  
#endif
	n-=1;
      }
      (pa->NumPair>1 && !ccd.Computing ? Show : Hide)(pa->buDelFun);
      for (b=FALSE, i=0; i<n; i++)
	(GetStatus(pa->cbJoin[i])&&Visible(pa->cbJoin[i]) ? (b=TRUE,Show) : Hide)(pa->rbAttrOf[i+1]);
      (b ? Show : Hide)(pa->grAttrOf);
      if (redraw) Redraw(pa->paSample);
      n=DIM(pa->rbAttrOf);
      if (pa->Kind==0)			/* current */
	SetStatus(pa->cbClear,(Boolean)(pa->Options.Flags&GWIN_CLEAR));
      else n-=1;
      for (i=0; i<n; i++)
	(Visible(pa->rbAttrOf[i])&&(j!=i+1) ? Show : Hide)(pa->buColorAs[i]);
      break;
    case 1:		/* axes */
      for (i=0; i<DIM(pa->Axis); i++) {
	for (j=0; j<DIM(pa->Axis[i].Prop); j++)
	  SetStatus(pa->Axis[i].cbProp[j],pa->Axis[i].Prop[j]);
	(pa->Axis[i].Prop[0] ? Show : Hide)(pa->Axis[i].cbProp[1]);	/* Tick depends on Limits */
      }
      SetStatus(pa->cbPalette,(Boolean)(pa->Options.Flags&GWIN_PALETTE));
      SetStatus(pa->cbCircle,(Boolean)(pa->Options.Flags&GWIN_CIRCLE));
      break;
  }
}

/* Add/Del functions button callbacks */
Local(void) AddFunProc(ButtoN bu) {
  WindowDataPtr pwd;
  GrAttrDataPtr pa;
  DrawAttr dattr;
  Int2 i;
  pa=GetWindowExtra(ParentWindow(bu));
  pwd=pa->Common.WinPtr;
  for (i=0; i<DIM(pa->Axis); i++) {
    pa->Axis[i].pTrace=_MemMore(pa->Axis[i].pTrace,ARRAYSIZE(CharPtr,pa->NumPair+1));
    pa->Axis[i].pTrace[pa->NumPair]=_StringSave("?");
  }
  pa->Attr=_MemMore(pa->Attr,ARRAYSIZE(FuncAttr,pa->NumPair+1));
  dattr.color=pwd->Foreground;
  dattr.width=2;
  dattr.style=-1;
  pa->Attr[pa->NumPair].Point=dattr;
  pa->Attr[pa->NumPair].Trace=dattr;
  dattr.style=1;
  pa->Attr[pa->NumPair].Path=dattr;
  if (pa->Kind==0) {		/* current */
    pwd->flags|=WFL_FUN_CHANGED;
    CorrectBarMax(pa->scBar,pa->NumPair);
  } else {			/* curve */
    pwd->Win.G.Attr=_MemMore(pwd->Win.G.Attr,ARRAYSIZE(FuncAttr,pa->NumPair+1));
    pwd->Win.G.FuncNum++;
  }
  pa->CurPair=pa->NumPair++;
  ShowAttributes(pa,TRUE);
}

Local(void) DelFunProc(ButtoN bu) {
  WindowDataPtr pwd;
  GrAttrDataPtr pa;
  Int2 i;
  pa=GetWindowExtra(ParentWindow(bu));
  pwd=pa->Common.WinPtr;
  /* it is always pa->NumPair>1; otherwise the button is hidden */
  if (pa->NumPair<=1) {
    /* 
       It is quite possible (in UNIX at least):
       press AddFunction twice and then QUICKLY
       press DelFunction three times.
    */
    return;
  }
  pa->NumPair--;
  for (i=0; i<DIM(pa->Axis); i++) {
    _MemFree(pa->Axis[i].pTrace[pa->CurPair]);
    MemMove(&pa->Axis[i].pTrace[pa->CurPair],
	    &pa->Axis[i].pTrace[pa->CurPair+1],
	    ARRAYSIZE(CharPtr,pa->NumPair-pa->CurPair));
    pa->Axis[i].pTrace=_MemMore(pa->Axis[i].pTrace,ARRAYSIZE(CharPtr,pa->NumPair));
  }
  MemMove(pa->Attr+pa->CurPair,pa->Attr+pa->CurPair+1,
	  ARRAYSIZE(FuncAttr,pa->NumPair-pa->CurPair));
  pa->Attr=_MemMore(pa->Attr,ARRAYSIZE(FuncAttr,pa->NumPair));
  if (pa->NumPair==pa->CurPair) pa->CurPair--;
  if (pa->Kind==0) {		/* current */	
    pwd->flags|=WFL_FUN_CHANGED;
  } else {			/* curve */
    pwd->Win.G.Attr=_MemMore(pwd->Win.G.Attr,ARRAYSIZE(FuncAttr,pa->NumPair));
    pwd->Win.G.FuncNum--;
  }
  ShowAttributes(pa,TRUE);
  if (pa->Kind==0) {		/* current */	
    CorrectBarMax(pa->scBar,pa->NumPair-1);	/* ShowAttributes() decreased scBar value */
  }
}

#if _WIN
#pragma argsused
#endif
/* V-func pair scroll bar callback */
Local(void) SelFuncPairProc(BaR bar, GraphiC graphic, Int2 new, Int2 old) {
  GrAttrDataPtr pa;
  if (new!=old) {
    pa=GetWindowExtra(ParentWindow(bar));
    pa->CurPair=new;
    ShowAttributes(pa,TRUE);
  }
}

/* cbJoin[0-2] button callback */
Local(void) JoinProc(ButtoN bu) {
  /* On/Off appropriate option in gSet group */
  GrAttrDataPtr pa;
  Int2 i;
  Boolean redraw=TRUE;
  pa=GetWindowExtra(ParentWindow(bu));
  for (i=0; i<DIM(pa->cbJoin); i++)
    if (bu==pa->cbJoin[i]) {
      switch (i) {
	case 0:		/* Join traces */
	  pa->Attr[pa->CurPair].Trace.style*=-1;
	  break;
	case 1:		/* Join paths */
	  pa->Attr[pa->CurPair].Path.style*=-1;
	  break;
	case 2:		/* Join shifts */
	  pa->Options.Shift.style*=-1;
	  redraw=FALSE;
	  break;
      }
      (GetStatus(bu) ? Show : Hide)(pa->rbAttrOf[i+1]);	/* +1 is for Points */
      ShowAttributes(pa,redraw);
      break;
    }
}

/* Palette/Clear/Circle common callback */
Local(void) PCCProc(ButtoN bu) {
  GrAttrDataPtr pa;
  Uint1 fl;
  pa=GetWindowExtra(ParentWindow(bu));
  if (bu==pa->cbPalette) fl=GWIN_PALETTE;
  if (bu==pa->cbClear) fl=GWIN_CLEAR;
  if (bu==pa->cbCircle) fl=GWIN_CIRCLE;
  if (GetStatus(bu)) pa->Options.Flags|=fl;
  else pa->Options.Flags&=~fl;
  Redraw(pa->paSample);
}


/* cbClear checkbox callback */
#define ClearProc PCCProc

/* cbPalette checkbox callback */
#define ShowPaletteProc PCCProc

/* cbCircle checkbox callback */
#define CircleProc PCCProc

/* grAttrOf callback */
Local(void) SetAttrOfProc(GrouP gr) {
  GrAttrDataPtr pa;
  pa=GetWindowExtra(ParentWindow(gr));
  ShowAttributes(pa,FALSE);
}

/* grWidth callback */
Local(void) WidthProc(GrouP grWidth) {
  GrAttrDataPtr pa;
  Uint1 width;
  pa=GetWindowExtra(ParentWindow(grWidth));
  width=GetValue(grWidth)-1;
  switch (GetValue(pa->grAttrOf)) {
    case 1:	/* Points */
      pa->Attr[pa->CurPair].Point.width=width;
      break;
    case 2:	/* Traces */
      pa->Attr[pa->CurPair].Trace.width=width;
      break;
    case 3:	/* Paths */
      pa->Attr[pa->CurPair].Path.width=width;
      break;
    case 4:	/* Shifts */
      pa->Options.Shift.width=width;
      break;
  }
  ShowAttributes(pa,TRUE);
}

/* grStyle callback */
Local(void) StyleProc(GrouP grStyle) {
  GrAttrDataPtr pa;
  Int1 style;
  pa=GetWindowExtra(ParentWindow(grStyle));
  style=GetValue(grStyle);
  switch (GetValue(pa->grAttrOf)) {
    case 1:	/* Points */
      pa->Attr[pa->CurPair].Point.style=style;
      break;
    case 2:	/* Traces */
      pa->Attr[pa->CurPair].Trace.style=style;
      break;
    case 3:	/* Paths */
      pa->Attr[pa->CurPair].Path.style=style;
      break;
    case 4:	/* Shifts */
      pa->Options.Shift.style=style;
      break;
  }
  ShowAttributes(pa,TRUE);
}

/* cbProp[1-5] callback */
Local(void) PropProc(ButtoN bu) {
  GrAttrDataPtr pa;
  Int2 i,j;
  pa=GetWindowExtra(ParentWindow(bu));
  for (i=0; i<DIM(pa->Axis); i++)
    for (j=0; j<DIM(pa->Axis[i].Prop); j++)
      if (bu==pa->Axis[i].cbProp[j]) {
	pa->Axis[i].Prop[j]=GetStatus(bu);
	Redraw(pa->paSample);
	break;
      }
}

/* Change options dependent from Min..Max option */
Local(void) LimitProc(ButtoN bu) {
  GrAttrDataPtr pa;
  Int2 i;
  pa=GetWindowExtra(ParentWindow(bu));
  i=pa->Axis[0].cbProp[0]==bu ? 0 : 1;
  (GetStatus(bu) ? Show : Hide)(pa->Axis[i].cbProp[1]);
  PropProc(bu);
}

/*-----------------*/
/* Color callbacks */

/* buColorAs[1-4] callback */
Local(void) ColorAsProc(ButtoN bu) {
  GrAttrDataPtr pa;
  Int2 i;
  Color col;
  pa=GetWindowExtra(ParentWindow(bu));
  for (i=0; i<DIM(pa->buColorAs); i++)
    if (bu==pa->buColorAs[i]) {
      switch (i) {
	case  0:	/* Points */
	  col=pa->Attr[pa->CurPair].Point.color;
	  break;      
	case  1:	/* Traces */
	  col=pa->Attr[pa->CurPair].Trace.color;
	  break;      
	case  2:	/* Path */
	  col=pa->Attr[pa->CurPair].Path.color;
	  break;      
	case  3:	/* Shifts */
	  col=pa->Options.Shift.color;
	  break;      
      }
      switch (GetValue(pa->grAttrOf)) {
	case 1:	/* Points */ 
	  pa->Attr[pa->CurPair].Point.color=col;
	  break;
	case 2:	/* Trace */
	  pa->Attr[pa->CurPair].Trace.color=col;
	  break;
	case 3:	/* Path */
	  pa->Attr[pa->CurPair].Path.color=col;
	  break;
	case 4:	/* Shift */
	  pa->Options.Shift.color=col;
      }
      ShowAttributes(pa,TRUE);
      break;
    }
}

/* Draw palette */
Local(void) PaletteProc(PaneL pnl) {
  RecT r;
  Int2 w,i,n;
  Select(pnl);
  GetPosition(pnl,&r);
  n=ColorNumber();
  if (!(w=(r.right-r.left)/n)) w=1;
  for (i=0; i<n; i++) {
    ColorSet(i);
    r.right=r.left+w+1;
    PaintRect(&r);
    r.left+=w;
  }
  GetPosition(pnl,&r);
  ColorSet(BlackIndex);
  FrameRect(&r);
}


/* Draw sample */

Local(CharPtr) SampLims[2][4]={
  {"-1","3",NULL,"1"},
  {"-2","4",NULL,"2"}
};

Local(void) SampleLimitsProc(PaneL paLimits) {
  GrAttrDataPtr pa;
  RecT r;
  Int2 i,j,off;
  pa=(GrAttrDataPtr)GetWindowExtra(ParentWindow(paLimits));
  Select(paLimits);
  SelectFont(SystemFont);
  GetPosition(paLimits,&r);
  for (i=0,off=r.left+2; i<DIM(pa->Axis); i++) {
    if (pa->Axis[i].bLimits) {
      r.left=off;
      off+=StringWidth(StrCpy(ParBuf,AttribTxt[i ? GAORD_P : GAABS_P]))+2;
      r.right=off;
      DrawString(&r,ParBuf,'l',FALSE);
      for (j=0; j<4; j++) {
	if (FuncIndex(j)) continue;
	if (j==3 && !pa->Axis[i].bTick) continue;
	r.left=off;
	off+=StringWidth(SampLims[i][j])+(sysCharWidth4);
	r.right=off;
	DrawString(&r,SampLims[i][j],'c',FALSE);
	FrameRect(&r);
	off+=2;
      }
    }
    off+=sysCharWidth2;
  }
}

#define SamplePaletteProc PaletteProc

#define GPN	5	/* number of G-points */
#define MPN	10	/* number of M-points in the first G-point */
#define MPR	1.0/(GPN)	/* radius of M-points in the first G-point */
#define MPRI	1.0/(GPN)	/* increment of radius */

Local(Int2) Ox,Oy,Dx,Dy;

#define GATHER	0	/* setting to 1 will create data file */

#if GATHER
#include <stdio.h>
  FILE *deb;
#endif

typedef struct {FloatLo cos,sin;} fPair;
Local(fPair) Data[GPN][MPN+1]={
  { {1,0},
    {0.809204,0.587528},
    {0.309623,0.950859},
    {-0.308108,0.951351},
    {-0.808267,0.588815},
    {-0.999999,0.00159255},
    {-0.810139,-0.586238},
    {-0.311137,-0.950365},
    {0.306593,-0.951841},
    {0.807329,-0.590102},
    {0.999995,-0.00318509}
  },
  { {0.951106,0.308866},
    {0.588172,0.808736},
    {0.000796274,1},
    {-0.586883,0.809672},
    {-0.950613,0.31038},
    {-0.951596,-0.30735},
    {-0.589459,-0.807798},
    {-0.00238894,-0.999997},
    {0.585593,-0.810606},
    {0.950117,-0.311893},
    {0.952085,0.305834}
  },
  { {0.809204,0.587528},
    {0.309623,0.950859},
    {-0.308108,0.951351},
    {-0.808267,0.588815},
    {-0.999999,0.00159255},
    {-0.810139,-0.586238},
    {-0.311137,-0.950365},
    {0.306593,-0.951841},
    {0.807329,-0.590102},
    {0.999995,-0.00318509},
    {0.811072,0.584947}
  },
  { {0.588172,0.808736},
    {0.000796274,1},
    {-0.586883,0.809672},
    {-0.950613,0.31038},
    {-0.951596,-0.30735},
    {-0.589459,-0.807798},
    {-0.00238894,-0.999997},
    {0.585593,-0.810606},
    {0.950117,-0.311893},
    {0.952085,0.305834},
    {0.590745,0.806859}
  },
  { {0.309623,0.950859},
    {-0.308108,0.951351},
    {-0.808267,0.588815},
    {-0.999999,0.00159255},
    {-0.810139,-0.586238},
    {-0.311137,-0.950365},
    {0.306593,-0.951841},
    {0.807329,-0.590102},
    {0.999995,-0.00318509},
    {0.811072,0.584947},
    {0.31265,0.949868}
  }
};

Local(void) GetCoord(Int2 Gp, Int2 Mp, Int2Ptr x, Int2Ptr y) {
#if GATHER
  FloatLo fi;
  fi=(1.57*Gp/GPN+6.28*Mp/MPN);
  *x=Ox+Dx*(MPR+Gp*MPRI)*cos(fi);
  *y=Oy+Dy*(MPR+Gp*MPRI)*sin(fi);
  fprintf(deb,"%2i %2i %g %g\n",Gp,Mp,cos(fi),sin(fi));
#else
  *x=Ox+Dx*(MPR+Gp*MPRI)*Data[Gp][Mp].cos;
  *y=Oy+Dy*(MPR+Gp*MPRI)*Data[Gp][Mp].sin;
#endif
}

Local(void) SampleDrawProc(PaneL paDraw) {
  GrAttrDataPtr pa;
  WindowDataPtr fool;
  DrawAttr flowers;
  WindoW attrwin;
  FuncAttr Attr;
  RecT r;
  Int2 num;
  Int2 i,j,ix,iy,ox,oy;
  pa=(GrAttrDataPtr)GetWindowExtra(attrwin=ParentWindow(paDraw));
  Select(paDraw);
  GetPosition(paDraw,&r);
  /* Use GraphDrawProc and VisualizerGPoint to draw the Sample */
  /* 1. Fool GraphDrawProc before call */
  fool=_MemNew(sizeof(WindowData));
  /* 2. Fill in *fool from *pa; *fool is filled by zeroes! */
  fool->flags|=WFL_FUN_CHANGED;
  num=pa->Kind;
  pa->Kind=0;
  GraphicGetAttributes(fool,attrwin);
  pa->Kind=num;
  MemMove(fool->Colors,pa->Common.Colors,sizeof(fool->Colors));
  for (i=0; i<DIM(fool->Win.G.Axis); i++)
    for (j=0; j<DIM(fool->Win.G.Axis[i].pText); j++) {
      _MemFree(fool->Win.G.Axis[i].pText[j]);
      fool->Win.G.Axis[i].pText[j]=_StringSave(SampLims[i][j]);
    }
  /* 3. Call GraphDrawProc */
  SetWindowExtra(attrwin,fool,NULL);
  Select(paDraw);
  GraphicDrawProc(paDraw);
  /* 4. Draw sampe curves */
  r=fool->Win.G.Slate;
  ClipRect(&r);
  Ox=(r.right+r.left)>>1;
  Oy=(r.bottom+r.top)>>1;
  Dx=(r.right-r.left+1)>>1;
  Dy=(r.bottom-r.top+1)>>1;
  Attr=pa->Attr[pa->CurPair];
  flowers.color=pa->Common.Foreground;
  flowers.width=1;
  flowers.style=1;
#if GATHER
  deb=fopen("deb","wt");
#endif
  for (j=0; j<GPN; j++) {		/* G-points */
    if (pa->Options.Flags&GWIN_CLEAR) {
      SetCurrentPen(&flowers);
      GraphicDrawProc(paDraw);
    }
    /* Join current point with the previous one along a trace */
    if (Attr.Trace.style>0 && IsDimGMore1()) {
      SetCurrentPen(&Attr.Trace);
      ix=Dx*(MPR+j*MPRI);
      iy=Dy*(MPR+j*MPRI);
      LoadRect(&r,Ox-ix-1,Oy-iy-1,Ox+ix+1,Oy+iy+1);
      FrameOval(&r);
    }
    SetCurrentPen(&Attr.Point);
    for (i=0; i<MPN; i++) {		/* its M-points */
      GetCoord(j,i,&ix,&iy);
      /* Join current point with previous one(s) along a path */
      if (Attr.Path.style>0) {
	if (j) {
	  GetCoord(j-1,i,&ox,&oy);
	  SetCurrentPen(&Attr.Path);
	  lLine(ix,iy,ox,oy);
	  SetCurrentPen(&Attr.Point);
	  lPoint(ox,oy);
	}
      }
/* It is not clear how to illustrate 'join along shift' */
      /* Join current point with the previous one along a shift */
/***************
      if (pa->Options.Shift.style>0) {
	SetCurrentPen(&pa->Options.Shift);
	ox=ix*1.1-0.1*Ox;
	oy=iy*1.1-0.1*Oy;
	lLine(ix,iy,ox,oy);
	SetCurrentPen(&Attr.Point);
	lPoint(ox,oy);
      }
**************/
      /* Plot the current point */
      lPoint(ix,iy);
    }
  }
  /* 5. Restore everything. This does not write but frees memory only */
  ((WindowDataPtr)GetWindowExtra(pa->Common.WinPtr->wiWindow))->ClassFunc->WriteDescription(fool,NULL);
  _MemFree(fool);
  SetWindowExtra(attrwin,pa,NULL);
#if GATHER
  fclose(deb);
#endif
}

Local(void) GrSampleProc(PaneL sample) {
  GrAttrDataPtr pa;
  RecT r;
  Int2 y;
  Int2 j;
  pa=(GrAttrDataPtr)GetWindowExtra(ParentWindow(sample));
  if (pa->Lock) return;
  pa->Lock=1;
  Hide(pa->paDraw);
  Hide(pa->paPalette);
  Hide(pa->paLimits);
  GetPosition(sample,&r);
  y=r.top;
  j=pa->SampleHeight;
  /* Limits sample */
  if (pa->Axis[0].bLimits || pa->Axis[1].bLimits) {
    GetPosition(pa->paLimits,&r);
    r.bottom+=y-r.top;
    r.top=y;
    SetPosition(pa->paLimits,&r);
    Show(pa->paLimits);
    j-=r.bottom-r.top;
    y+=r.bottom-r.top;
  }
  /* Palette sample */
  if (pa->Options.Flags&GWIN_PALETTE) {
    GetPosition(pa->paPalette,&r);
    r.bottom+=y-r.top;
    r.top=y;
    SetPosition(pa->paPalette,&r);
    Show(pa->paPalette);
    j-=r.bottom-r.top;
    y+=r.bottom-r.top;
  }
  /* Draw sample */
  GetPosition(pa->paDraw,&r);
  r.top=r.bottom-j;
  SetPosition(pa->paDraw,&r);
  Show(pa->paDraw);
  pa->Lock=0;
}


/* Select color */
Local(void) ClickProc(PaneL pnl, PoinT pt) {
  GrAttrDataPtr pa;
  RecT r;
  Color col;
  pa=(GrAttrDataPtr)GetWindowExtra(ParentWindow(pnl));
  GetPosition(pnl,&r);
  col=floor((pt.x-r.left)/((r.right-r.left)/ColorNumber()));
  switch (pa->Kind) {
    case 0:	/* current attributes */
    case 2:	/* curve attributes */
      switch (GetValue(pa->grAttrOf)) {
	case 1:	/* Points */ 
	  pa->Attr[pa->CurPair].Point.color=col;
	  break;
	case 2:	/* Trace */
	  pa->Attr[pa->CurPair].Trace.color=col;
	  break;
	case 3:	/* Path */
	  pa->Attr[pa->CurPair].Path.color=col;
	  break;
	case 4:	/* Shift */
	  pa->Options.Shift.color=col;
      }
      break;
    case 1:	/* axes attributes */
      switch (GetValue(pa->grColor)) {
	case 1:	/* foreground */
	  pa->Common.Foreground=col;
      	  break;
	case 2:	/* background */
	  pa->Common.Background=col;
	  break;
      }
  }
  Redraw(pa->paSample);
}

/*---------------------------------------------------------------*/
/* Functions related to processing of attributes of active curve */

Local(void) ActivateSegment(GrAttrDataPtr pa, Int2 s) {
  WindowDataPtr pwd;
  Int2 i,k;
  Pointtype ptype;	/* local code of the type of a special point */
  pwd=pa->Common.WinPtr;
  pa->spcur=s;
  BSSeek(pa->spbs[s-1],0,SEEK_SET);
  RestoreVisualAttr(&DiagramLib,pa->spbs[s-1],NULL);
  if (!pwd->Win.G.FuncNum) {
    /* This window was created AFTER the active curve was stored/edited */
    pwd->Win.G.FuncNum=pwd->Win.G.SaveFuncNum;
    pwd->Win.G.Attr=_MemNew(ARRAYSIZE(FuncAttr,pwd->Win.G.FuncNum));
    MemMove(pwd->Win.G.Attr,pwd->Win.G.SaveAttr,ARRAYSIZE(FuncAttr,pwd->Win.G.FuncNum));
  }
  pa->NumPair=pwd->Win.G.FuncNum;
  pa->CurPair=0;
  /* Copy V-functions (for show sample only!) */
  for (i=0; i<DIM(pa->Axis); i++) {
    pa->Axis[i].pTrace=_MemNew(ARRAYSIZE(CharPtr,pa->NumPair));
    for (k=0; k<pa->NumPair; k++)
      pa->Axis[i].pTrace[k]=_StringSave(k>=pwd->Win.G.SaveFuncNum ?
					"?" :
					pwd->Win.G.Axis[i].Funcs[k].pFunc);
  }
#if MYSWITCH
  SwitchSetParams(pa->swAttr,1,pa->NumPair);  
#else
  SetSwitchParams(pa->swAttr,1,pa->NumPair);  
#endif
  /* Copy attributes of V-functions */
  pa->Attr=_MemNew(ARRAYSIZE(FuncAttr,pa->NumPair));
  MemMove(pa->Attr,pwd->Win.G.Attr,ARRAYSIZE(FuncAttr,pa->NumPair));
  /* Show endpoints' type */
  if (s==1) SetTitle(pa->prFrom,AttribTxt[GAFP]);
  else {
    DataLibSeek(&DiagramLib,pa->splist+s-2);
    DataLibRead(&DiagramLib,(CharPtr)&ptype,sizeof(ptype));
    SetTitle(pa->prFrom,GetSPname(ptype,TRUE));
  }
  if (s==pa->spnum) SetTitle(pa->prTo,AttribTxt[GALP]);
  else {
    DataLibSeek(&DiagramLib,pa->splist+s-1);
    DataLibRead(&DiagramLib,(CharPtr)&ptype,sizeof(ptype));
    SetTitle(pa->prTo,GetSPname(ptype,TRUE));
  }
}

Local(void) DeactivateSegment(GrAttrDataPtr pa, Int2 s) {
  WindowDataPtr pwd;
  Int2 i,k;
  pwd=pa->Common.WinPtr;
  BSFree(pa->spbs[s-1]);
  pa->spbs[s-1]=BSNew(0);
  MemMove(pwd->Win.G.Attr,pa->Attr,ARRAYSIZE(FuncAttr,pa->NumPair));
  for (i=0; i<DIM(pa->Axis); i++)
    if (pa->Axis[i].pTrace) {
      for (k=0; k<pa->NumPair; k++)
	_MemFree(pa->Axis[i].pTrace[k]);
      pa->Axis[i].pTrace=_MemFree(pa->Axis[i].pTrace);
    }
  pa->Attr=_MemFree(pa->Attr);
  SaveVisualAttr(&DiagramLib,NULL,pa->spbs[s-1],TRUE);
}

/* Segment number callback */
#if MYSWITCH
Local(void) SegmentProc(GraphiC sw, Int2 new, Int2 old) {
#else
Local(void) SegmentProc(SwitcH sw, Int2 new, Int2 old) {
#endif
  GrAttrDataPtr pa;
  pa=(GrAttrDataPtr)GetWindowExtra(ParentWindow(sw));
  DeactivateSegment(pa,old);
  ActivateSegment(pa,new);
  ShowAttributes(pa,TRUE);
}

#define AddAttrProc AddFunProc

#define DelAttrProc DelFunProc

/* Attributes number callback */
#if _WIN
#pragma argsused
#endif
#if MYSWITCH
Local(void) AttrProc(GraphiC sw, Int2 new, Int2 old) {
#else
Local(void) AttrProc(SwitcH sw, Int2 new, Int2 old) {
#endif
  GrAttrDataPtr pa;
  pa=(GrAttrDataPtr)GetWindowExtra(ParentWindow(sw));
  pa->CurPair=new-1;
  ShowAttributes(pa,TRUE);
}

/*-----------------------------------------*/
/* Copy attributes from/to WindowData area */

Local(void) GrCopyToWork(GrAttrDataPtr pa, WindowDataPtr pwd) {
  Int2 i,k;
  if (pa->Kind==2) {		/* attribs of active curve */
    PushVisualAttr(TRUE);	/* will be restored in GraphicFreeAttributes */
    pa->dlsave=DataLibSavePos(&DiagramLib);
    ReadSPList(&pa->spnum,&pa->splist);
#if MYSWITCH
    SwitchSetParams(pa->swSegment,1,pa->spnum);
#else
    SetSwitchParams(pa->swSegment,1,pa->spnum);
#endif
    ReadVAList(pa->spnum,&pa->spbs);
    ActivateSegment(pa,1);
  } else {
    /* Get memory for local editable copies of attributes */
    pa->NumPair=pwd->Win.G.FuncNum;
    pa->CurPair=pwd->Win.G.FuncCur;
    /* Copy attributes of V-functions */
    pa->Attr=_MemNew(ARRAYSIZE(FuncAttr,pa->NumPair));
    MemMove(pa->Attr,pwd->Win.G.Attr,ARRAYSIZE(FuncAttr,pa->NumPair));
    /* Copy V-functions */
    for (i=0; i<DIM(pa->Axis); i++) {
      pa->Axis[i].pTrace=_MemNew(ARRAYSIZE(CharPtr,pa->NumPair));
      for (k=0; k<pa->NumPair; k++)
	pa->Axis[i].pTrace[k]=_StringSave(pwd->Win.G.Axis[i].Funcs[k].pFunc);
    }
  }
  for (i=0; i<DIM(pa->Axis); i++)
    MemMove(pa->Axis[i].Prop,pwd->Win.G.Axis[i].Prop,sizeof(pa->Axis[i].Prop));
  /* Copy general attributes */
  MemMove(pa->Common.Colors,pwd->Colors,sizeof(pa->Common.Colors));
  pa->Options=pwd->Win.G.Options;
}

/* Get attributes */
Global(void) GraphicGetAttributes(WindowDataPtr pwd, WindoW attrwin) {
  GrAttrDataPtr pa;
  Int2 i,j,k;
  pa=(GrAttrDataPtr)GetWindowExtra(attrwin);
  if (pa->Kind==2) {		/* attribs of active curve */
    /* First save current segment's attributes to spbs[...] */
    DeactivateSegment(pa,pa->spcur);
    /* Now save to DiagramLib */
    WriteVAList(pa->spnum,pa->spbs);
    /* Restore everything */
    PopVisualAttr();
    DataLibRestorePos(&DiagramLib,pa->dlsave);
    return;
  }
  if (pwd->flags&WFL_FUN_CHANGED) {	/* a func was changed */
    /* free old pair descriptors */
    for (i=0; i<DIM(pwd->Win.G.Axis); i++) {
      for (k=0; k<pwd->Win.G.FuncNum; k++)
	_MemFree(pwd->Win.G.Axis[i].Funcs[k].pFunc);
      _MemFree(pwd->Win.G.Axis[i].Funcs);
    }
    _MemFree(pwd->Win.G.Attr);
    /* Create new ones */
    pwd->Win.G.FuncNum=pa->NumPair;
    for (i=0; i<DIM(pwd->Win.G.Axis); i++) {
      pwd->Win.G.Axis[i].Funcs=_MemNew(ARRAYSIZE(AxisFunc,pwd->Win.G.FuncNum));
      for (k=0; k<pwd->Win.G.FuncNum; k++) {
	StrTrim(pa->Axis[i].pTrace[k]);
	if (StrLen(pa->Axis[i].pTrace[k])==0) {
	  CharPtr name;
	  Int2 dim;
	  _MemFree(pa->Axis[i].pTrace[k]);
	  GetNameAndDim(0,1,&name,&dim);
	  sprintf(ParBuf,"%s%s",name,dim==1 ? "" : "[0]");
	  pa->Axis[i].pTrace[k]=_StringSave(ParBuf);
	  myWarning("VI_EMPTY",ParBuf);
	}
	pwd->Win.G.Axis[i].Funcs[k].pFunc=_StringSave(pa->Axis[i].pTrace[k]);
      }
    }
    pwd->Win.G.Attr=_MemNew(ARRAYSIZE(FuncAttr,pwd->Win.G.FuncNum));
  }
  MemMove(pwd->Win.G.Attr,pa->Attr,ARRAYSIZE(FuncAttr,pwd->Win.G.FuncNum));
  for (i=0; i<DIM(pwd->Win.G.Axis); i++)
    MemMove(pwd->Win.G.Axis[i].Prop,pa->Axis[i].Prop,sizeof(pwd->Win.G.Axis[i].Prop));
  pwd->Win.G.FuncCur=pa->CurPair;
  pwd->Win.G.Options=pa->Options;
  switch (pa->Kind) {
    case 0:		/* current */
    case 2:		/* curve */
      break;
    case 1:		/* axes */
      for (i=0; i<DIM(pwd->Win.G.Axis); i++)
	for (j=0; j<DIM(pwd->Win.G.Axis[i].pText); j++)
	  if (!FuncIndex(j))
	    GetTitleAndSave(pa->Axis[i].teText[j],&pwd->Win.G.Axis[i].pText[j]);
      break;
  }
}

/* Free memory */
Global(void) GraphicFreeAttributes(WindoW attrwin) {
  GrAttrDataPtr pa;
  Int2 i,k;
  pa=(GrAttrDataPtr)GetWindowExtra(attrwin);
  if (pa->Kind==2) {
    FreeSPList(pa->spnum,pa->splist);
    FreeVAList(pa->spnum,pa->spbs);
    if (!pa->Common.Ok) {	/* GraphicGetAttributes was not called, i.e. Cancel */
      PopVisualAttr();
      DataLibRestorePos(&DiagramLib,pa->dlsave);
    }
  }
  for (i=0; i<DIM(pa->Axis); i++)
    if (pa->Axis[i].pTrace) {
      for (k=0; k<pa->NumPair; k++)
	_MemFree(pa->Axis[i].pTrace[k]);
      _MemFree(pa->Axis[i].pTrace);
    }
  _MemFree(pa->Attr);
}

Local(void) CreateSampleGroup(GrouP parent, Int2 width, Int2 height) {
  GrAttrDataPtr pa;
  GrouP g;
  pa=(GrAttrDataPtr)GetWindowExtra(ParentWindow(parent));
  g=CreateNormalGroup(parent,0,0,GAWINSAMP_P);
  pa->SampleHeight=height;
  pa->paSample=SimplePanel(g,1,1,GrSampleProc);
  Hide(pa->paLimits=SimplePanel(g,width,sysLineHeight,SampleLimitsProc));
  Hide(pa->paPalette=SimplePanel(g,width,sysLineHeight,SamplePaletteProc));
  Hide(pa->paDraw=SimplePanel(g,width,height,SampleDrawProc));
}

/*---------------------------------*/
/* Ask attributes                  */
/*  num=0/1/2 - current/axes/curve */
Global(void) GraphicAskAttributes(WindowDataPtr pwd, Int2 num) {
  WindoW w;
  GrAttrDataPtr pa;
  GrouP gFunctions,gAJB,gA,gB,gJ,gL,gSWSC,gSWS,gOptions;
  GrouP g,gT,gAxes,gAS,gS,gColor,gFBSS,gFBS;
  GrouP MainGroup;
  PaneL pPalette;
  ButtoN bu;
  LisT list=NULL;
  RecT r,rr;
  PoinT pt;
  Int2 i,j;
  Char buf[3];
  /* Create attribute window and get misc sizes */
  w=CreateAttrWin(GWINATTR,sizeof(GrAttrData));
  MainGroup=InitLargeWindow(w);
  SetGroupSpacing(MainGroup,0,sysLineHeight2);
  SetGroupMargins(MainGroup,sysCharWidth2,sysLineHeight2);
  pa=GetWindowExtra(w);
  pa->Lock=0;
  pa->Kind=num;		/* functions/axes */
  pa->Common.FuncChange=GrFuncChanged;
  pa->Common.WinPtr=pwd;
  sscanf(AttribTxt[GASIZES],"%i,%i,(%i,%i)",&numwidth,&textwidth,&lines,&linelen);
  switch (num) {
    case 0:		/* current attributes  */
    case 2:		/* curve attributes */
      if (num==2) {	/* attribs of active curve */
	g=CreateHiddenGroup(MainGroup,2,0,1);
	StdPrmt=FALSE;
#if MYSWITCH
	StaticPrompt(g,AttribTxt[GASEG_P],0,SwitchHeight(),NULL,'l');
	pa->swSegment=SwitchCreate(g,10,SegmentProc);
#else
	CreatePrompt(g,GASEG_P);
	pa->swSegment=LeftRightSwitch(g,TRUE,SegmentProc);
#endif
	CreatePrompt(g,GASPF_P);
	pa->prFrom=CreatePrompt(g,-1);
	CreatePrompt(g,GASPT_P);
	pa->prTo=CreatePrompt(g,-1);
	StdPrmt=TRUE;
      } else {
	/* V-functions group */
	gFunctions=CreateNormalGroup(MainGroup,-2,0,GACURVES_P);
	  gAJB=CreateHiddenGroup(gFunctions,0,3,1);
	    gA=CreateHiddenGroup(gAJB,3,0,0);
	      CreatePrompt(gA,GAABS_P);
	      pa->Axis[0].teFunc=DialogText(gA,"",textwidth,GrFuncChanged);
	      SetTextSelect(pa->Axis[0].teFunc,MarkSelection,NULL);
	      GetNextPosition(gA,&pt);
	      pt.x-=hSX+1;
	      SetNextPosition(gA,pt);
	      pa->scBar=ScrollBar(gA,0,1,SelFuncPairProc);
	      CorrectBarMax(pa->scBar,pwd->Win.G.FuncNum-1);	/* [0..FuncNum-1] */
	      CreatePrompt(gA,GAORD_P);
	      pa->Axis[1].teFunc=DialogText(gA,"",textwidth,GrFuncChanged);
	      SetTextSelect(pa->Axis[1].teFunc,MarkSelection,NULL);
	    gB=CreateHiddenGroup(gAJB,2,0,1);
	      SetGroupMargins(gB,hMX,4*nMY);
	      bu=PushButton(gB,GetButtonTitle(AttribTxt[GAADDFUN_B]),AddFunProc);
	      if (ccd.Computing) Hide(bu);
	      pa->buDelFun=PushButton(gB,GetButtonTitle(AttribTxt[GADELFUN_B]),DelFunProc);
	    GetPosition(gAJB,&r);
	    i=r.right-r.left;
	    GetPosition(pa->buDelFun,&rr);
	    i-=rr.right-rr.left;
	    GetPosition(bu,&rr);
	    i-=rr.right-rr.left;
	    if (i>0) {
	      i/=3;
	      GetPosition(bu,&r);
	      OffsetRect(&r,i-hMX,0);
	      SetPosition(bu,&r);
	      GetPosition(pa->buDelFun,&r);
	      OffsetRect(&r,2*i-hSX,0);
	      SetPosition(pa->buDelFun,&r);
	    }
	  gL=CreateHiddenGroup(gFunctions,1,0,0);
	    list=CreateNamesList(gL,linelen,lines,NamesListProc,FALSE);
      }
      pa->grAttributes=CreateNormalGroup(MainGroup,0,10,GAATTR_P);
      if (num==2) {	/* attribs of active curve */
	g=CreateHiddenGroup(pa->grAttributes,4,0,0);
	StdPrmt=FALSE;
#if MYSWITCH
	StaticPrompt(g,AttribTxt[GAAN_P],0,SwitchHeight(),NULL,'l');
	pa->swAttr=SwitchCreate(g,10,AttrProc);
#else
	CreatePrompt(g,GAAN_P);
	pa->swAttr=LeftRightSwitch(g,TRUE,AttrProc);
#endif
	GetNextPosition(g,&pt);
	pt.x+=sysCharWidth;
	SetNextPosition(g,pt);
	bu=PushButton(g,GetButtonTitle(AttribTxt[GAAA_B]),AddAttrProc);
	if (ccd.Computing) Hide(bu);
	pa->buDelFun=PushButton(g,GetButtonTitle(AttribTxt[GADA_B]),DelAttrProc);	
	StdPrmt=TRUE;
      }
	gJ=CreateNormalGroup(pa->grAttributes,2,0,GAJOIN_T);
	  pa->cbJoin[1]=CheckBox(gJ,AttribTxt[GAPATHS_P],JoinProc);
	  if (IsDimGMore1())
	    pa->cbJoin[0]=CheckBox(gJ,AttribTxt[GATRACES_P],JoinProc);
	gSWSC=CreateHiddenGroup(pa->grAttributes,2,0,0);
	  gSWS=CreateHiddenGroup(gSWSC,0,3,1);
	    pa->grAttrOf=CreateNormalGroup(gSWS,2,0,GAATTROF_P);
	    SetAction(pa->grAttrOf,(ActnProc)SetAttrOfProc);
	    pa->rbAttrOf[0]=RadioButton(pa->grAttrOf,AttribTxt[GAPOINTS_P]);
	    pa->rbAttrOf[1]=RadioButton(pa->grAttrOf,AttribTxt[GATRACES_P]);
	    pa->rbAttrOf[2]=RadioButton(pa->grAttrOf,AttribTxt[GAPATHS_P]);
	    if (pa->Kind==0)		/* current */
	      pa->rbAttrOf[3]=RadioButton(pa->grAttrOf,AttribTxt[GASHIFTS_P]);
	    SetValue(pa->grAttrOf,3);
	    pa->grWidth=CreateNormalGroup(gSWS,4,0,GAWIDTH_P);
	    SetAction(pa->grWidth,(ActnProc)WidthProc);
	    for (i=0; i<8; i++) {
	      sprintf(buf,"%i",(int)i); 
	      RadioButton(pa->grWidth,buf);
	    }
	    pa->grStyle=CreateNormalGroup(gSWS,2,0,GASTYLE_P);
	    SetAction(pa->grStyle,(ActnProc)StyleProc);
	    RadioButton(pa->grStyle,AttribTxt[GASOLID_P]);
	    RadioButton(pa->grStyle,AttribTxt[GADOTTED_P]);
	    RadioButton(pa->grStyle,AttribTxt[GADASHED_P]);
	  GetPosition(gSWS,&r);
	  i=r.bottom-r.top-2*nMY-sysLineHeight;
	  CreateSampleGroup(gSWSC,5*i/4,i);
	GetPosition(gSWSC,&rr);
	pPalette=SimplePanel(pa->grAttributes,rr.right-rr.left,sysLineHeight<<1,PaletteProc);
	SetPanelClick(pPalette,ClickProc,NULL,NULL,NULL);
	g=CreateNormalGroup(pa->grAttributes,4,0,GACOLORAS_P);
	pa->buColorAs[0]=PushButton(g,AttribTxt[GAPOINTS_P],ColorAsProc);
	pa->buColorAs[1]=PushButton(g,AttribTxt[GATRACES_P],ColorAsProc);
	pa->buColorAs[2]=PushButton(g,AttribTxt[GAPATHS_P],ColorAsProc);
      if (num==0) {	/* current attribs */
	pa->buColorAs[3]=PushButton(g,AttribTxt[GASHIFTS_P],ColorAsProc);
	gOptions=CreateNormalGroup(MainGroup,2,0,GAOPTIONS_P);
	  pa->cbClear=CheckBox(gOptions,AttribTxt[GACLEAR_P],ClearProc);
	  pa->cbJoin[2]=CheckBox(gOptions,AttribTxt[GAJOINSHIFT_P],JoinProc);
	AlignObjects(ALIGN_RIGHT,(HANDLE)pa->grAttributes,(HANDLE)gOptions,NULL);
      } else {
	AlignObjects(ALIGN_RIGHT,(HANDLE)pa->prFrom,(HANDLE)pa->prTo,
				 (HANDLE)pPalette,NULL);
	gL=NULL;
      }
      AlignObjects(ALIGN_LOWER,(HANDLE)pa->Axis[1].teFunc,(HANDLE)pa->scBar,NULL);
      AlignObjects(ALIGN_RIGHT,(HANDLE)pa->paLimits,(HANDLE)pa->paPalette,(HANDLE)pa->paDraw,
			       (HANDLE)list,
			       NULL);
      AlignObjects(ALIGN_RIGHT,(HANDLE)Parent(pa->paLimits),
			       (HANDLE)pPalette,Parent(pa->buColorAs[0]),gJ,
			       (HANDLE)gL,
			       NULL);
      g=pa->grAttributes;
      break;
    case 1:		/* attributes of axes */
      g=CreateHiddenGroup(MainGroup,0,4,1);
	gT=CreateHiddenGroup(g,2,0,0);
	CreatePrompt(gT,WINTITLE_P);
	pa->Common.teWinTitle=DialogText(gT,pwd->pWinTitle,textwidth,NULL);
	gAxes=CreateHiddenGroup(g,2,0,0);
	for (i=0; i<DIM(pa->Axis); i++) {
	  gAS=CreateNormalGroup(gAxes,0,2,i==0 ? GAABS_P : GAORD_P);
	    gA=CreateHiddenGroup(gAS,3,0,0);
	    StdPrmt=FALSE;
	    CreatePrompt(gA,GAMIN_P);
	    CreatePrompt(gA,GAMAX_P);
	    CreatePrompt(gA,GATICK_P);
	    StdPrmt=TRUE;
	    for (j=0; j<DIM(pa->Axis[i].teText); j++)
	      if (!FuncIndex(j))
		pa->Axis[i].teText[j]=DialogText(gA,pwd->Win.G.Axis[i].pText[j],numwidth,NULL);
	    gS=CreateNormalGroup(gAS,3,0,GAOPT_P);	/* Show in window */
	    sprintf(ParBuf,"%s..%s",AttribTxt[GAMIN_P],AttribTxt[GAMAX_P]);
	    pa->Axis[i].cbProp[0]=CheckBox(gS,ParBuf,LimitProc);
	    pa->Axis[i].cbProp[1]=CheckBox(gS,AttribTxt[GATICK_P],PropProc);
	    pa->Axis[i].cbProp[2]=CheckBox(gS,AttribTxt[GASCALE_P],PropProc);
	    pa->Axis[i].cbProp[3]=CheckBox(gS,AttribTxt[GAFUNC_P],PropProc);
	    pa->Axis[i].cbProp[4]=CheckBox(gS,AttribTxt[GAGRID_P],PropProc);
	    pa->Axis[i].cbProp[5]=CheckBox(gS,AttribTxt[GAAXIS_P],PropProc);
	}
	pa->cbCircle=CheckBox(g,AttribTxt[GACIRCLE_P],CircleProc);
	gColor=CreateNormalGroup(g,0,2,PALETTE_P);
	  gFBSS=CreateHiddenGroup(gColor,2,0,0);
	    gFBS=CreateHiddenGroup(gFBSS,0,2,1);
	      pa->grColor=CreateNormalGroup(gFBS,0,2,COL_P);	/* 'Select color of' */
	      RadioButton(pa->grColor,AttribTxt[FG_P]);
	      RadioButton(pa->grColor,AttribTxt[BG_P]);
	      SetValue(pa->grColor,1);
	      pa->cbPalette=CheckBox(gFBS,AttribTxt[SHOWPAL_P],ShowPaletteProc);
	      GetPosition(gFBS,&r);
	      CreateSampleGroup(gFBSS,1,(r.bottom-r.top)<<1);
	      pPalette=SimplePanel(gColor,1,sysLineHeight<<1,PaletteProc);
	      SetPanelClick(pPalette,ClickProc,NULL,NULL,NULL);
	AlignObjects(ALIGN_RIGHT,(HANDLE)pa->Axis[1].cbProp[2],
				 (HANDLE)pa->Axis[1].cbProp[5],
				 (HANDLE)pa->Axis[1].teText[3],
				 (HANDLE)pa->paLimits,
				 (HANDLE)pa->paPalette,
				 (HANDLE)pa->paDraw,
				 NULL);
	AlignObjects(ALIGN_RIGHT,(HANDLE)Parent(pa->paDraw),
				 (HANDLE)pPalette,
				 NULL);
	AlignObjects(ALIGN_RIGHT,(HANDLE)gAxes,
				 (HANDLE)pa->Common.teWinTitle,
				 NULL);
      break;
  }
  /* Fill in work area (temporary copies of attrributes) */
  GrCopyToWork(pa,pwd);
  /* Create terminate buttons */
  CreateButtons((WindoW)MainGroup,g);
  /* That's all */
  ShowAttributes(pa,TRUE);
  ShowLargeWindow(MainGroup);
}

#undef teMin
#undef teMax
#undef teFunc
#undef teTick

/*******************/
/* Text attributes */
/*******************/

/* Stores data while attribute input/change */
typedef struct {
  /* common fields */
  CommonData Common;	/* Common data for all classes of windows */
  PaneL paSample;	/* Sample panel */
  Color bg,fg;
  /* text window's fields */
  TexT teText[6];
#define teHistl	teText[0]	/* Max number of lines in history */
#define teHistb	teText[1]	/* history begins */
#define teHiste	teText[2]	/* history ends */
#define teList	teText[3]	/* List of expressions */
#define teFormat	teText[4]	/* Output format */
#define teTitle	teText[5]	/* Title of table */
  ButtoN TxtMsg;	/* CheckBox for 'Show messages' */
  Int2 OldTextLen;
} TeAttrData, PNTR TeAttrDataPtr;

/* Get attributes */
Global(void) TextGetAttributes(WindowDataPtr pwd, WindoW attrwin) {
  TeAttrDataPtr pa;
  Int2 i;
  pa=(TeAttrDataPtr)GetWindowExtra(attrwin);
  for (i=0; i<DIM(pwd->Win.T.pText); i++) {
    GetTitleAndSave(pa->teText[i],&pwd->Win.T.pText[i]);
  }
  StrTrim(pwd->Win.T.pList);
  pwd->Win.T.ShowMsg=GetStatus(pa->TxtMsg);
}

/* Free memory */
#if _WIN
#pragma argsused
#endif
Global(void) TextFreeAttributes(WindoW attrwin) {
}

/* Set WFL_FUN_CHANGED flag */
Local(void) TeFuncChanged(TexT t) {
  WindowDataPtr pwd;
  TeAttrDataPtr pa;
  pa=(TeAttrDataPtr)GetWindowExtra(ParentWindow(t));
  pwd=pa->Common.WinPtr;
  pwd->flags|=WFL_FUN_CHANGED;
}

/* Title has been changed */
Local(void) TeTitleChanged(TexT t) {
  TeAttrDataPtr pa;
  Int2 TitleLen;
  pa=(TeAttrDataPtr)GetWindowExtra(ParentWindow(t));
  TitleLen=(Int2)TextLength(pa->teTitle);
  if (!(pa->OldTextLen*TitleLen)) Redraw(pa->paSample);
  pa->OldTextLen=TitleLen;
}

/* Select color */
Local(void) TextClickProc(PaneL pnl, PoinT pt) {
  TeAttrDataPtr pa;
  RecT r;
  Color col;
  pa=(TeAttrDataPtr)GetWindowExtra(ParentWindow(pnl));
  GetPosition(pnl,&r);
  col=floor((pt.x-r.left)/((r.right-r.left)/ColorNumber()));
  pa->Common.Foreground=col;
  Redraw(pa->paSample);
}

Local(void) TeSampleProc(PaneL sample) {
  FloatLo x;
  TeAttrDataPtr pa;
  RecT r;
  Int2 i,lh;
  pa=(TeAttrDataPtr)GetWindowExtra(ParentWindow(sample));
  Select(sample);
  SelectFont(programFont);
  lh=LineHeight();
  GetPosition(sample,&r);
  if (TextLength(pa->teTitle)) {
    ColorSet(pa->Common.Foreground);
    MoveTo(r.left,r.top+lh);
    PaintString(" x      x^2      x^3     x^4");
    r.top+=lh;
  }
  ClipRect(&r);
  InvertColors();
  PaintRect(&r);
  InvertColors();
  ColorSet(pa->Common.Foreground);
  for (i=0,x=0.1; i<10; i++, x+=0.1) {
    MoveTo(r.left,r.top+=lh);
    PaintText("%3.1f%9.4f%9.4f%9.4f",x,x*x,x*x*x,x*x*x*x);
  }
}

Local(Int2) TTitle[]={	/* NB! Indices must be in the same order as in pText[] */
  TAHISTL_P,
  TAHISTB_P,
  TAHISTE_P,
  TALIST_P,
  TAFORMAT_P,
  TATITLE_P
};

/* Ask attributes */
#if _WIN
#pragma argsused
#endif
Global(void) TextAskAttributes(WindowDataPtr pwd, Int2 num) {
  WindoW w;
  GrouP g,gt,gl,gColor;
  GrouP MainGroup;
  PaneL paPalette;
  LisT list;
  TeAttrDataPtr pa;
  RecT r,rr;
  PoinT pt;
  Int2 i,h;
  /* Create attribute window and get misc sizes */
  w=CreateAttrWin(TWINATTR,sizeof(TeAttrData));
  MainGroup=InitLargeWindow(w);
  SetGroupSpacing(MainGroup,0,sysLineHeight2);
  SetGroupMargins(MainGroup,sysCharWidth2,sysLineHeight2);
  pa=GetWindowExtra(w);
  pa->Common.WinPtr=pwd;
  pa->Common.FuncChange=TeFuncChanged;
  sscanf(AttribTxt[TASIZES],"%i,(%i,%i)",&textwidth,&lines,&linelen);
  g=CreateHiddenGroup(MainGroup,0,4,1);
  MemMove(pa->Common.Colors,pwd->Colors,sizeof(pa->Common.Colors));
  /* Create text input fields with prompts */
  gt=CreateHiddenGroup(g,2,0,1);
  CreatePrompt(gt,WINTITLE_P);
  pa->Common.teWinTitle=DialogText(gt,pwd->pWinTitle,textwidth,NULL);
  for (i=0; i<DIM(pwd->Win.T.pText); i++) {
    CreatePrompt(gt,TTitle[i]);
    pa->teText[i]=DialogText(gt,pwd->Win.T.pText[i],textwidth,NULL);
  }
  SetAction(pa->teList,(ActnProc)TeFuncChanged);
  SetAction(pa->teTitle,(ActnProc)TeTitleChanged);
  pa->OldTextLen=(Int2)TextLength(pa->teTitle);
  pa->Common.teInsertTo=pa->teList;
  pa->TxtMsg=CheckBox(gt,AttribTxt[TAMSG_B],NULL);
  SetStatus(pa->TxtMsg,pwd->Win.T.ShowMsg);
  gl=CreateHiddenGroup(g,2,0,0);
  /* Create list of visible variable names */
  list=CreateNamesList(gl,linelen,lines,NamesListProc,FALSE);
  GetPosition(g,&r);
  GetPosition(list,&rr);
  GetNextPosition(gl,&pt);
  gt=CreateNormalGroup(gl,0,0,GAWINSAMP_P);
  SelectFont(programFont);
  h=(rr.bottom-rr.top)/FontHeight()*FontHeight();
  pa->paSample=SimplePanel(gt,r.right-pt.x-2*nMX,h,TeSampleProc);
  /* Create palette */
  gColor=CreateNormalGroup(g,0,2,PALETTE_P);
  paPalette=SimplePanel(gColor,1,sysLineHeight<<1,PaletteProc);
  SetPanelClick(paPalette,TextClickProc,NULL,NULL,NULL);
  AlignObjects(ALIGN_RIGHT,(HANDLE)pa->paSample,(HANDLE)paPalette,NULL);
  AlignObjects(ALIGN_LOWER,(HANDLE)Parent(list),(HANDLE)gt,NULL);
  /* Create terminate buttons */
  CreateButtons((WindoW)MainGroup,g);
  /* Show and activate */
  ShowLargeWindow(MainGroup);
  SelectText(pa->Common.teInsertTo,1000,1000);
}

#undef teList
#undef teFormat
#undef teTitle



/*************************/
/* Coordiante attributes */
/*************************/

/* Stores data while attribute input/change */
typedef struct {
  /* common fields */
  CommonData Common;	/* Common data for all classes of windows */
  PaneL paSample;	/* Sample panel */
  Color bg,fg;
  LisT Names;
  PairPtr PNTR wSelection;
  Uint2Ptr wRealIds;
  Int2 wClassNum;
  FloatHiPtr PNTR wPrevVals;
#if MYSWITCH
  Switch sPrec;
#else
  SwitcH sPrec;
#endif
  ButtoN bAlign;
} CoAttrData, PNTR CoAttrDataPtr;

/* Get attributes */
#if _WIN
#pragma argsused
#endif
Global(void) CoordGetAttributes(WindowDataPtr pwd, WindoW attrwin) {
  CoAttrDataPtr coPa=GetWindowExtra(attrwin);
  MultiNamesList(NULL,NULL);
#if MYSWITCH
  pwd->Win.C.prec=SwitchGetValue(coPa->sPrec);
#else
  pwd->Win.C.prec=GetValue(coPa->sPrec);
#endif
  pwd->Win.C.align=GetStatus(coPa->bAlign) ? 'r' : 'l';
}

/* Free memory */
#if _WIN
#pragma argsused
#endif
Global(void) CoordFreeAttributes(WindoW attrwin) {
}

#if _WIN
#pragma argsused
#endif
#if MYSWITCH
Local(void) PrecProc(GraphiC sw, Int2 new, Int2 old) {
#else
Local(void) PrecProc(SwitcH sw, Int2 new, Int2 old) {
#endif
  CoAttrDataPtr pa;
  pa=(CoAttrDataPtr)GetWindowExtra(ParentWindow(sw));
  Redraw(pa->paSample);
}

Local(void) AlignProc(ButtoN align) {
  CoAttrDataPtr pa;
  pa=(CoAttrDataPtr)GetWindowExtra(ParentWindow(align));
  Redraw(pa->paSample);
}

/* Select color */
Local(void) CoordClickProc(PaneL pnl, PoinT pt) {
  CoAttrDataPtr pa;
  RecT r;
  Color col;
  pa=(CoAttrDataPtr)GetWindowExtra(ParentWindow(pnl));
  GetPosition(pnl,&r);
  col=floor((pt.x-r.left)/((r.right-r.left)/ColorNumber()));
  pa->Common.Foreground=col;
  Redraw(pa->paSample);
}

Local(void) CoSampleProc(PaneL sample) {
  FloatLo x;
  CoAttrDataPtr pa;
  RecT r;
  int prec;
  Int2 i,lh;
  Boolean align;
  pa=(CoAttrDataPtr)GetWindowExtra(ParentWindow(sample));
  Select(sample);
  SelectFont(SystemFont);
  lh=FontHeight();
  GetPosition(sample,&r);
  ColorSet(pa->Common.Foreground);
  MoveTo(r.left,r.top+lh-sysDescent);
  PaintString("Special point");
  r.top+=lh;
  ClipRect(&r);
  InvertColors();
  PaintRect(&r);
  InvertColors();
  ColorSet(pa->Common.Foreground);
#if MYSWITCH
  prec=(int)SwitchGetValue(pa->sPrec);
#else
  prec=(int)GetValue(pa->sPrec);
#endif
  align=GetStatus(pa->bAlign);
  for (i=0,x=0.12345678910111213; i<10; i++, x+=0.1111111111111111) {
    MoveTo(r.left,r.top+=lh);
    PaintText("%c %s%.*g","xyzuvwabcd"[i],align ? "          " : "",prec,x);
  }
}

/* EnumClasses callback */

Local(CoAttrDataPtr) coPa;

#if _WIN
#pragma argsused
#endif
Local(void) ClassCallback(LisT dummy, CharPtr title, CharPtr name,
			  Boolean defvis, Uint2 realId) {
  WindowDataPtr coPwd=coPa->Common.WinPtr;
  PairPtr pp;
  Int2 n=coPa->wClassNum,i,j,m;
  if (coPa->wClassNum++) {
    coPa->wSelection=_MemMore(coPa->wSelection,ARRAYSIZE(PairPtr,coPa->wClassNum));
    coPa->wRealIds=_MemMore(coPa->wRealIds,ARRAYSIZE(Uint2,coPa->wClassNum));
    coPa->wPrevVals=_MemMore(coPa->wPrevVals,ARRAYSIZE(FloatHiPtr,coPa->wClassNum));
  } else {
    coPa->wSelection=_MemNew(ARRAYSIZE(PairPtr,coPa->wClassNum));
    coPa->wRealIds=_MemNew(ARRAYSIZE(Uint2,coPa->wClassNum));
    coPa->wPrevVals=_MemNew(ARRAYSIZE(FloatHiPtr,coPa->wClassNum));
  }
  coPa->wRealIds[n]=realId;
  coPa->wSelection[n]=NULL;
  coPa->wPrevVals[n]=NULL;
  for (i=0; i<coPwd->Win.C.ClassNum; i++)
    if (coPwd->Win.C.RealIds[i]==realId) {
      pp=coPwd->Win.C.Selection[i];
      if (pp) {
	for (j=m=0; m+=ABS(pp[j].num), pp[j++].num>0; );
	coPa->wSelection[n]=(PairPtr)_MemNew(ARRAYSIZE(Pair,j));
	MemCopy(coPa->wSelection[n],pp,ARRAYSIZE(Pair,j));
	coPa->wPrevVals[n]=(FloatHiPtr)_MemNew(ARRAYSIZE(FloatHi,m));
	MemCopy(coPa->wPrevVals[n],coPwd->Win.C.PrevVals[i],ARRAYSIZE(FloatHi,m));
      }
      return;
    }
  if (defvis) {
    pp=coPa->wSelection[n]=(PairPtr)_MemNew(ARRAYSIZE(Pair,1));
    /* pp->from is already 0 */
    pp->num=-*GetClassDim(name);
  }
}

/* Ask attributes */
#if _WIN
#pragma argsused
#endif
Global(void) CoordAskAttributes(WindowDataPtr pwd, Int2 num) {
  WindoW w;
  GrouP g,gt,gColor,gw;
  GrouP MainGroup;
  PaneL paPalette;
  CoAttrDataPtr pa;
  RecT r;
  Int2 h,i;
  sscanf(AttribTxt[COSIZES],"%i",&numwidth);
  /* Create attribute window and get misc sizes */
  w=CreateAttrWin(CWINATTR,sizeof(CoAttrData));
  MainGroup=InitLargeWindow(w);
  SetGroupSpacing(MainGroup,0,sysLineHeight2);
  SetGroupMargins(MainGroup,sysCharWidth2,sysLineHeight2);
  g=CreateHiddenGroup(MainGroup,2,0,1);
  pa=GetWindowExtra(w);
  pa->Common.WinPtr=pwd;
  MemMove(pa->Common.Colors,pwd->Colors,sizeof(pa->Common.Colors));
  /* Recreate list of visible attrib and preserve selection data.
     The list might be changed due to manipulation of point-curve type */
  pa->wClassNum=0;
  coPa=pa;
  EnumClasses(0,ClassCallback);
  for (i=0; i<pwd->Win.C.ClassNum; i++) {
    _MemFree(pwd->Win.C.Selection[i]);
    _MemFree(pwd->Win.C.PrevVals[i]);
  }
  _MemFree(pwd->Win.C.PrevVals);
  _MemFree(pwd->Win.C.Selection);
  _MemFree(pwd->Win.C.RealIds);
  pwd->Win.C.ClassNum=pa->wClassNum;
  pwd->Win.C.Selection=pa->wSelection;
  pwd->Win.C.RealIds=pa->wRealIds;
  pwd->Win.C.PrevVals=pa->wPrevVals;
  /* Create list of visible variable names */
  MultiNamesList(g,pwd->Win.C.Selection);
  /* Format */
  gt=CreateNormalGroup(MainGroup,2,0,COFORM_T);
  gw=CreateHiddenGroup(gt,2,0,0);
#if MYSWITCH
  StaticPrompt(gw,AttribTxt[COPREC_P],0,SwitchHeight(),NULL,'l');
  pa->sPrec=SwitchCreate(gw,DBL_DIG,PrecProc);
  SwitchSetValue(pa->sPrec,pwd->Win.C.prec);
#else
  StdPrmt=FALSE;
  CreatePrompt(gw,COPREC_P);
  pa->sPrec=LeftRightSwitch(gw,TRUE,PrecProc);
  SetSwitchParams(pa->sPrec,pwd->Win.C.prec,DBL_DIG);
  StdPrmt=TRUE;
#endif
  gw=CreateHiddenGroup(gt,2,0,0);
  pa->bAlign=CheckBox(gw,AttribTxt[COALIGN_B],AlignProc);
  if (pwd->Win.C.align=='r') SetStatus(pa->bAlign,TRUE);
  /* Sample */
  GetPosition(g,&r);
  gt=CreateNormalGroup(MainGroup,0,0,GAWINSAMP_P);
  h=((r.bottom-r.top)>>1)/sysFontHeight*sysFontHeight;
  pa->paSample=SimplePanel(gt,r.right-2*nMX,h,CoSampleProc);
  /* Create palette */
  gColor=CreateNormalGroup(MainGroup,0,2,PALETTE_P);
  paPalette=SimplePanel(gColor,1,sysLineHeight<<1,PaletteProc);
  SetPanelClick(paPalette,CoordClickProc,NULL,NULL,NULL);
  AlignObjects(ALIGN_RIGHT,(HANDLE)pa->paSample,(HANDLE)paPalette,NULL);
  /* Create terminate buttons */
  CreateButtons((WindoW)MainGroup,MainGroup);
  /* Show and activate */
  ShowLargeWindow(MainGroup);
}

#undef teList
#undef teFormat
#undef teTitle
#undef teFile
#undef teLines

#endif	/* WD */